/**  
  *  IHS_to_RGB Version 1.0, 2008-10-10, by 
  *  Martin Schlueter, i3mainz, Mainz University of Applied Sciences, Germany
  *
  *  This ImageJ plugin performs a color transformation from IHS to RGB.
  *  
  *  Based on
  *  Wolfgang GOEPFERT:
  *  Raumbezogene Informationssysteme
  *  Textbook, Wichmann, Karlsruhe, 1987.
  *
  *  To learn more please refer to the related web pages with examples and
  *  applications.
  *
  *  For consultance and practical applications concerning Mobile 3D Coordinate
  *  Measuring Techniques, 3D Digitizing, Deformation Measurement and Analysis 
  *  please contact
  *  Martin Schlueter: schlueter(at)geoinform.fh-mainz.de   
  *  
  *  Copyright (c) [2008-10-10] by Prof. Dr.-Ing. Martin Schlueter, i3mainz,
  *  Mainz University of Applied Sciences, Germany

  *  Permission is hereby granted, free of charge, to any person obtaining a 
  *  copy of this software and associated documentation files (the "Software"), 
  *  to deal in the Software without restriction, including without limitation 
  *  the rights to use, copy, modify, merge, publish, distribute, sublicense, 
  *  and/or sell copies of the Software, and to permit persons to whom the 
  *  Software is furnished to do so, subject to the following conditions:
  *  The above copyright notice and this permission notice shall be included in
  *  all copies or substantial portions of the Software.
  *
  *  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  *  IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
  *  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL 
  *  THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
  *  LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING 
  *  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER 
  *  DEALINGS IN THE SOFTWARE.
  *
  *  OSI Certified Open Source Software
  *
  * History
  * 2008/10/10: First version online
  *
  */

import ij.ImagePlus;
import ij.plugin.filter.PlugInFilter;
import ij.process.ImageProcessor;

public class IHS_to_RGB implements PlugInFilter {

    public int setup(String arg, ImagePlus imp) {
        return DOES_RGB+DOES_STACKS;
    }
    
    public void run(ImageProcessor ip) { 
        int div = 255 * 255 * 3; 
        int add = div >> 1;  // add = div /2 by rightshift

        int[] pixels = (int[]) ip.getPixels();

        //iterate over all pixels
        for (int j = 0; j < pixels.length; j++) {

            //get int-packed color pixel
            int c = pixels[j];

            //extract RGB components from color pixel
            int i = (c & 0xff0000) >> 16;
            int h = (c & 0x00ff00) >> 8;
            int s = (c & 0x0000ff);

            i = 3 * i;
            h = 3 * h;
                
            //red, green, blue
            int r = 0, g = 0, b = 0, e = 0;
            int d = 255 * (255 - s);

            if (h <= 255) {
               e = 3 * s * h;
               r = i * (65025 + 510 * s - e);
               g = i * (d + e);
               b = i * d;
            }   
            else if (h <= 510) {
               e = 3 * s * (h - 255);
               r = i * d;
               g = i * (65025 + 510 * s - e);
               b = i * (d + e);
            }
            else {
               e = 3 * s * (h - 510);
               r = i * (d + e);
               g = i * d;
               b = i * (65025 + 510 * s - e);
            }

            //re-scale r, g, b to [0-255]; add half of divisor to keep round-off errors small
            r = Math.min(255, Math.max(0, ((r + add) / div)));
            g = Math.min(255, Math.max(0, ((g + add) / div)));
            b = Math.min(255, Math.max(0, ((b + add) / div)));

            // reassemble color pixel
            c = ((r & 0xff)<<16) | ((g & 0xff)<<8) | b & 0xff; 
            pixels[j] = c;
        }
    }
}
